/**
 * Copyright (c) 2005 JavaPlant Software.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.javaplant.portlets;

import java.util.ArrayList;
import java.util.Map;
import java.util.Collections;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author Brad Smith <mvcportlet@gmail.com>
 *         Date: Oct 9, 2005
 */
public class RSSFeedConfiguration {

    private Log log = LogFactory.getLog(this.getClass());

    private ArrayList rssURLs = new ArrayList();
    private String xsl;
    private String httpProxyPort;
    private String httpProxyHost;
    private int timeout = -1;// -1 = never timeout

    public List getRssURLs() {
        return Collections.unmodifiableList(rssURLs);
    }

    public void setRssURLs(List rssURLs) {
        this.rssURLs.clear();
        this.rssURLs.addAll(rssURLs);
    }

    public final void addEmptyRow() {
        this.rssURLs.add("");
    }

    public final boolean removeRow(int rowNumber) {
        try {
            this.rssURLs.remove(rowNumber);
            return true;
        } catch (IndexOutOfBoundsException e) {

        }
        return false;
    }

    public String getRssURL(int index) {
        return (index < rssURLs.size()) ? (String)rssURLs.get(index) : "";
    }

    public final void setRssURL(int index, String value) {
        while (rssURLs.size() < index + 1) {
            rssURLs.add(null);
        }
        rssURLs.set(index, value.trim());
    }

    public String getXsl() {
        return xsl;
    }

    public void setXsl(String xsl) {
        this.xsl = xsl;
    }

    public String getHttpProxyPort() {
        return httpProxyPort;
    }

    public void setHttpProxyPort(String httpProxyPort) {
        this.httpProxyPort = httpProxyPort;
    }

    public String getHttpProxyHost() {
        return httpProxyHost;
    }

    public void setHttpProxyHost(String httpProxyHost) {
        this.httpProxyHost = httpProxyHost;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    // - Utility Methods ---
    public static void setProperties(RSSFeedConfiguration configuration, Map properties) {
        if (configuration != null && properties != null && properties.size()>0) {
            if (properties.containsKey("httpProxyHost")) {
                String [] tmp = getPropertyValue(properties,"httpProxyHost");
                if (tmp.length>0) {
                    configuration.httpProxyHost=tmp[0];
                }
            }
            if (properties.containsKey("httpProxyPort")) {
                String [] tmp = getPropertyValue(properties,"httpProxyPort");
                if (tmp.length>0) {
                    configuration.httpProxyPort=tmp[0];
                }
            }
            if (properties.containsKey("xsl")) {
                String [] tmp = getPropertyValue(properties,"xsl");
                if (tmp.length>0) {
                    configuration.xsl=tmp[0];
                }
            }
            if (properties.containsKey("timeout")) {
                String [] tmp = getPropertyValue(properties,"timeout");
                if (tmp.length>0) {
                    try {
                        configuration.timeout = Integer.parseInt(tmp[0]);
                    } catch(Exception e) {

                    }
                }
            }
            if (properties.containsKey("rssURLs")) {
                String [] tmp = getPropertyValue(properties,"rssURLs");
                if (tmp.length>0) {
                    configuration.rssURLs.clear();
                    for (int i=0;i<tmp.length;i++) {
                        configuration.rssURLs.add(tmp[i]);
                    }
                }
            }
        }
    }

    public final Map getPreferencesMap() {
        final Map map = new HashMap(5);
        // TODO : clean this up - all validation (checking) should occur in a validator method
        // timeout
        map.put("timeout", String.valueOf(this.timeout));
        // httpProxyHost
        String tmp = (this.httpProxyHost != null && !"".equals(this.httpProxyHost)) ? this.httpProxyHost : null;
        map.put("httpProxyHost",tmp);
        // httpProxyPort
        tmp = (this.httpProxyPort != null && !"".equals(this.httpProxyPort)) ? this.httpProxyPort : null;
        map.put("httpProxyPort",tmp);
        map.put("xsl",this.xsl);
        // "rssURLs"
        // remove empty strings and duplicates
        final List tmpRssUrls = new ArrayList(this.rssURLs.size());
        for (Iterator it=this.rssURLs.iterator();it.hasNext();) {
            final String tmpRssURL = (String)it.next();
            if (tmpRssURL == null || "".equals(tmpRssURL) || tmpRssUrls.contains(tmpRssURL)) continue;
            tmpRssUrls.add(tmpRssURL);
        }
        map.put("rssURLs",tmpRssUrls.toArray(new String[tmpRssUrls.size()]));
        return map;
    }

    private static String [] getPropertyValue(Map map, String key) {
        String [] result = null;
        if (map != null && map.size()>0) {
            Object tmp = map.get(key);
            if (tmp !=null) {
                if (tmp instanceof String[]) {
                    result = (String [])tmp;
                } else {
                    result = new String[1];
                    result[0] = tmp.toString();
                }
            }
        }
        return result;
    }

    public String toString() {
        final StringBuffer buf = new StringBuffer();
        buf.append("RSSFeedConfiguration");
        buf.append("{log=").append(log);
        buf.append(",rssURLs=").append(rssURLs);
        buf.append(",xsl=").append(xsl);
        buf.append(",httpProxyPort=").append(httpProxyPort);
        buf.append(",httpProxyHost=").append(httpProxyHost);
        buf.append(",timeout=").append(timeout);
        buf.append('}');
        return buf.toString();
    }
}
