/**
 * Copyright (c) 2005 JavaPlant Software.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.javaplant.portlet;

import java.io.IOException;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/** A base class for portlets that provides a variety of behaviors and properties.
 * Form bean behaviors
 *  -Automatic form-bean creation and population from request parameters.
 *  -a callback for validation of a form bean
 *  -form bean is placed in PortletRequest (RenderRequest) scope
 * 
 * @author Brad Smith <mvcportlet@gmail.com>
 *         Date: Aug 14, 2005
 */
public abstract class BaseMVCPortlet extends GenericPortlet {

    protected final Log log = LogFactory.getLog(this.getClass());

    /** PortletConfiguration */
    private PortletConfiguration portletConfiguration = null;

    /** This method creates the PortletConfiguration for this portlet based on the
     * &lt;init-param&gt;'s in portlet.xml.  It then invokes the template method
     * <b>doInit()</b> which subclasses should override for custom portlet initialization.
     *
     * @throws PortletException
     */
    public final void init() throws PortletException {
        final PortletConfig config = getPortletConfig();
        portletConfiguration = PortletUtils.getPortletConfiguration(config.getPortletContext(), config);
        if (portletConfiguration.isShowConfiguration()) {
            getPortletContext().log(PortletUtils.showInitParams(config));
        }
        final String portletForm = config.getInitParameter("portlet.form");
        if (portletForm == null || "".equals(portletForm)) {
            log.warn(getPortletName()+" extends BasePortlet but does not declare a portlet.form as an init-param in portlet.xml");
        }
        // Invoke subclass-specific doInit() method
        doInit();
    }

    /** Template method - subclasses should override and provide custom portlet initialization.
     *
     * @throws PortletException
     */
    protected void doInit() throws PortletException {}

   /** Standard Portlet API method.  This method cannot be overridden.  The BaseMVCPortlet
    * provides the implementation.  Applications <i>must</i> provide an implementation of
    * {@link #doProcessAction(PortletForm, ActionRequest, ActionResponse) doProcessAction}.
    *
    * @param request
    * @param response
    * @throws PortletException
    * @throws IOException
    */
    public final void processAction(ActionRequest request, ActionResponse response)
        throws PortletException, IOException {
        if (log.isDebugEnabled()) {
            log.debug("processAction() called...");
            log.debug("ActionRequest.getParameterMap() :" + request.getParameterMap());
            log.debug("ActionRequest.getParameterNames() :" + request.getParameterNames());
        }
        if (portletConfiguration != null) {
            try {
                // Create a PortletForm - populate it's properties using request parameters.
                PortletForm portletForm = getForm();
                if (log.isDebugEnabled()) {
                    log.debug("PortletForm.action="+portletForm.getAction());
                }
                if (portletForm == null) {
                    log.info("PortletForm not available for this Portlet - "+getPortletName());
                    log.info("Your doProcessAction() method will not be called.");
                    return;
                } else {
                    portletForm.setResourceBundle(getPortletConfig().getResourceBundle(request.getLocale()));
                    log.debug(portletForm);
                }
                portletForm = PortletUtils.populatePortletForm(portletForm, request);

                if (portletForm.validate()) {
                    // call subclass's doProcessAction(...)
                    doProcessAction(portletForm, request, response);
                    // Add the portletForm to the PortletProcessingContext and store as a session attribute
                    // - I discovered during development that placing attributes in portlet-request scope
                    //   during the processAction(...) does not guarantee they will be available in
                    //   portlet-request scope during render processing.
                    PortletProcessingContext context = new PortletProcessingContext();
                    context.setActionForm(portletForm);
                    context.setActionPortletName(getPortletName());
                    final PortletSession session = request.getPortletSession();
                    session.setAttribute("com.javaplant.portlet.PortletProcessingContext", context);
                    log.debug("context->" + session.getAttribute("com.javaplant.portlet.PortletProcessingContext"));
                } else {
                    // do error handling....
                }
            } catch (Exception e) {
                log.error(e,e);
                throw new PortletException(e.getMessage(),e);
            }
        } else {
            log.warn("No configuration has been supplied for portlet "+getPortletName());
        }
    }

    /** This method enforces the BaseMVCPortlet-<i>render</i> lifecycle.  You do not override this method.</i>.
     *
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public final void render(RenderRequest request, RenderResponse response)
        throws PortletException, IOException {

        if (log.isDebugEnabled()) {
            log.debug("BaseMVCPortlet.render()");
            log.debug("Getting user....");
            log.debug("Locale - " + request.getLocale());
        }
        // Form Bean juggling...Keep the 'action' form 'separate' from the render form
        final PortletSession session = request.getPortletSession();
        final PortletProcessingContext context = (PortletProcessingContext)session.getAttribute("com.javaplant.portlet.PortletProcessingContext");
        session.removeAttribute("com.javaplant.portlet.PortletProcessingContext");
        PortletForm form = null;
        if (context == null || !context.getActionPortletName().equals(getPortletName())) {
            form = getForm();
            if (log.isDebugEnabled()) {
                log.debug("created a new PortletForm -> "+form);
            }
        } else { // this is the portlet that responded to the action event
                 //   - no need to create the PortletForm again
            form = context.getActionForm();
            if (log.isDebugEnabled()) {
                log.debug("using existing (action)PortletForm -> "+form);
            }
            form.setResourceBundle(getResourceBundle(request.getLocale()));
            if (context != null) {
                request.removeAttribute("com.javaplant.portlet.PortletProcessingContext");
            }
        }
        request.setAttribute(PortletForm.KEY /*getFormBeanKey(response)*/,form);

        // This is taken from the javax.portlet.GenericPortlet implementation of render(...).
        // ---the goal is to preserve the same behavior as GenericPortlet by default
        response.setTitle(getTitle(request));
        if (log.isDebugEnabled()) {
            //log.debug("Generated form key="+getFormBeanKey(response));
            log.debug("render() - PortletForm.getClass().getName()=" + ((form == null) ? "NO FORM DEFINED" : form.getClass().getName()));
            log.debug("renderRequest() : PortletForm=" + form);
        }
        PortletPage portletPage = null;
        WindowState state = request.getWindowState();
        if(!state.equals(WindowState.MINIMIZED)) {
            PortletMode mode = request.getPortletMode();
            if(mode.equals(PortletMode.VIEW)) {
                portletPage = doViewRender(form, request, response);
            } else if(mode.equals(PortletMode.EDIT)) {
                portletPage = doEditRender(form, request, response);
            } else if(mode.equals(PortletMode.HELP)) {
                portletPage = doHelpRender(form, request, response);
            } else {
                throw new PortletException("unknown portlet mode: " + mode+"\n or unknown portlet action: "+form.getAction());
            }
        }
        // Generate portlet content by including a page's content
        if (portletPage!= null && portletPage.getPagePath()!= null) {
            response.setContentType(portletPage.getContentType());
            PortletRequestDispatcher dispatcher = this.getPortletContext().getRequestDispatcher(portletPage.getPagePath());
            dispatcher.include(request, response);
        }
    }

    /** Subclasses must provide an implementation of this method to handle portlet actions.
     *
     * @param form
     * @param request
     * @param response
     * @throws PortletException
     * @throws java.io.IOException
     */
    protected abstract void doProcessAction(PortletForm form, ActionRequest request, ActionResponse response)
        throws PortletException, IOException;

    /** Template method - subclasses should override and provide custom VIEW-render processing if necessary.
     * The default behavior is to return the JSP page associated with the <b>portlet.jsp.view</b> init-param
     * in portlet.xml
     *
     * @param form
     * @param request
     * @param response
     * @throws PortletException
     * @throws java.io.IOException
     */
    protected PortletPage doViewRender(PortletForm form, RenderRequest request, RenderResponse response)
        throws PortletException, IOException {
        return new PortletPage(getViewPage());
    }

    /** Template method - subclasses should override and provide custom EDIT-render processing if necessary.
     * The default behavior is to return the JSP page associated with the <b>portlet.jsp.edit</b> init-param
     * in portlet.xml
     *
     * @param form
     * @param request
     * @param response
     * @throws PortletException
     * @throws java.io.IOException
     */
    protected PortletPage doEditRender(PortletForm form, RenderRequest request, RenderResponse response)
        throws PortletException, IOException {
        return new PortletPage(getEditPage());
    }

    /** Template method - subclasses should override and provide custom HELP-render processing if necessary.
     * The default behavior is to return the JSP page associated with the <b>portlet.jsp.help</b> init-param
     * in portlet.xml
     *
     * @param form
     * @param request
     * @param response
     * @throws PortletException
     * @throws java.io.IOException
     */
    protected PortletPage doHelpRender(PortletForm form, RenderRequest request, RenderResponse response)
        throws PortletException, IOException {
        return new PortletPage(getHelpPage());
    }

    /** Returns the path to the JSP page that is configured for this portlet using
     * <b>portlet.jsp.view</b>.
     *
     * @return path to the <i>VIEW</i>-JSP page (null if not configured for this portlet).
     */
    protected final String getViewPage() {
        return this.portletConfiguration.getViewPage();
    }

    /** Returns the path to the JSP page that is configured for this portlet using
     * <b>portlet.jsp.edit</b>.
     *
     * @return path to the <i>EDIT</i>-JSP page (null if not configured for this portlet).
     */
    protected final String getEditPage() {
        return this.portletConfiguration.getEditPage();
    }

    /** Returns the path to the JSP page that is configured for this portlet using
     * <b>portlet.jsp.help</b>.
     *
     * @return path to the <i>HELP</i>-JSP page (null if not configured for this portlet).
     */
    protected final String getHelpPage() {
        return this.portletConfiguration.getHelpPage();
    }

    /** Get the path to an arbitrary JSP page component that is part of this portlet's
     *  configuration.
     *
     * @param pageName name of page to get from PortletConfiguration.
     * @return path to requested JSP page (null if page is not configured for portlet).
     */
    protected final String getPage(String pageName) {
        return this.portletConfiguration.getPage(pageName);
    }

    /** Create a new instance of the PortletForm configured for this portlet.
     *
     * @return PortletForm for this portlet (null if class not found or not configured for portlet).
     */
    protected final PortletForm getForm() {
        PortletForm portletForm = null;
        try {
            if (portletConfiguration != null && portletConfiguration.getPortletFormClass() != null) {
                portletForm = (PortletForm)portletConfiguration.getPortletFormClass().newInstance();
            }
        } catch (Exception e) {
            log.error(e, e);
        }
        return portletForm;
    }

    /** Add errors to PortletForm, set page to display, etc...
     *
     * @param form
     * @throws PortletException
     * @throws IOException
     */
    /** NOT USED YET
    protected abstract void doValidationFailed(PortletForm form)
        throws PortletException, IOException;
    */

}
