/**
 * Copyright (c) 2005 JavaPlant Software.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.javaplant.portlet;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author Brad Smith <mvcportlet@gmail.com>
 *         Date: Aug 14, 2005
 */
public class PortletUtils {

    private static final Log log = LogFactory.getLog(PortletUtils.class);

    public static final PortletForm populatePortletForm(PortletForm portletForm, PortletRequest request)
        throws PortletException {
        // Set the corresponding properties of our bean
        try {
            BeanUtils.populate(portletForm, request.getParameterMap());
        } catch(Exception e) {
            log.error(e,e);
            throw new PortletException("There was a problem populating BindPortletFormTag from request.\n"+e.getMessage(), e);
        }
        return portletForm;
    }

    public static final PortletConfiguration getPortletConfiguration(PortletContext portletContext, PortletConfig portletConfig)
        throws PortletException {
        final PortletConfiguration portletConfiguration = new PortletConfiguration();

        // move all params to a map....it's easier to access
        final Map jspFrags = new HashMap();
        Enumeration initParamsEnum = portletConfig.getInitParameterNames();
        while (initParamsEnum.hasMoreElements()) {
            String name = (String)initParamsEnum.nextElement();
            if (name.startsWith("portlet.jsp.")) {
                jspFrags.put(name,portletConfig.getInitParameter(name));
            }
        }

        // Get PortletForm class and validate it is available in class path
        String portletFormName = portletConfig.getInitParameter("portlet.form");
        if (portletFormName != null) { // validate PortletForm existence...
            try {
                final Class portletFormClass = Class.forName(portletFormName);
                // Test instantiation...
                final Object portletForm = portletFormClass.newInstance();
                if (portletForm == null) { // huh?
                    // if somehow we get to this point and portletForm is null, then we have a problem
                    throw new ClassNotFoundException(portletFormName+" not found.");
                }
                portletConfiguration.setPortletFormClass(portletFormClass);
            } catch (Exception e) {
                throw new PortletException("ERROR:  Cannot find or create class named " + portletFormName + ".\n" + e.getMessage(), e);
            }
        } else {
            portletContext.log("WARNING:  "+portletConfig.getPortletName()
                                +" extends com.javaplant.portlet.BaseMVCPortlet but does not declare a PortletForm\n"
                                +"(using portlet.form <init-param> in portlet.xml).");
        }

        // Locate view fragments declared in portlet.xml.
        for (Iterator it=jspFrags.keySet().iterator();it.hasNext();) {
            String jspFragmentName = (String)it.next();
            String jspFragmentURI = (String)jspFrags.get(jspFragmentName);
            if (validateJspViewFragment(portletContext,jspFragmentName,jspFragmentURI)) {
                if ("portlet.jsp.view".equals(jspFragmentName)) {
                    portletConfiguration.setViewPage(jspFragmentURI);
                } else if ("portlet.jsp.edit".equals(jspFragmentName)) {
                    portletConfiguration.setEditPage(jspFragmentURI);
                } else if ("portlet.jsp.help".equals(jspFragmentName)) {
                    portletConfiguration.setHelpPage(jspFragmentURI);
                } else {
                    // add page to generic map of pages
                    portletConfiguration.setPage(jspFragmentName.substring("portlet.jsp.".length()),jspFragmentURI);
                }
            }
        }
        final String showConfig = portletConfig.getInitParameter("portlet.showConfiguration");
        portletConfiguration.setShowConfiguration("true".equalsIgnoreCase(showConfig)
                                                   || "yes".equalsIgnoreCase(showConfig)
        );
        return portletConfiguration;
    }

    public static final boolean validateJspViewFragment(PortletContext portletContext,
                                                String jspFragmentName, String jspFragmentURI)
        throws PortletException {
        // validate view fragment
        if (jspFragmentName != null && !"".equals(jspFragmentName)
              && jspFragmentURI != null && !"".equals(jspFragmentURI)) {
            PortletRequestDispatcher reqDisp = portletContext.getRequestDispatcher(jspFragmentURI);
            if (reqDisp == null) {
                log.warn("Portlet init-param, "+jspFragmentName+", which refers to '"+jspFragmentURI+"' is not valid.");
                return false;
            }
            return true;
        }
        return false;
    }

    public final static String createWWWFormURLEncoded(String s) {
        try {
            return URLEncoder.encode(s, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error(e,e);
            throw new Error("UTF-8 encoding missing");
        }
    }

    // Checks the PortletRequest to see if user is in role.
    public static final boolean isUserInRole(String role, PortletRequest request) {
        return request.isUserInRole(role);
    }

    /** Generates a report displaying the &lt;init-param&gt;'s for this this portlet (from portlet.xml).
     *
     * @param config
     * @return String containing generated report of portlet configuration.
     */
    public static final String showInitParams(PortletConfig config) {
        final StringBuffer sb = new StringBuffer();
        sb.append("\n\n================================================================================\n");
        sb.append("Portlet Name=").append(config.getPortletName()).append("\n");
        sb.append("--Init Params-------------------------------------------------------------------\n");
        for (Enumeration enum = config.getInitParameterNames();enum.hasMoreElements();) {
            final String name = (String)enum.nextElement();
            final String value = config.getInitParameter(name);
            sb.append(name).append("=").append(value).append("\n");
        }
        sb.append("================================================================================\n\n");
        return sb.toString();
    }

}

